@@ -6,6 +6,11 @@ class JsonUtils:
 
     @staticmethod
     def is_valid_json(json_str):
+        """
+        Verify if a string is a valid JSON.
+        :param json_str: The string to verify.
+        :return: True if the string is a valid JSON, False otherwise.
+        """
         try:
             json.loads(json_str)
             return True
@@ -14,6 +19,12 @@ class JsonUtils:
 
     @staticmethod
     def has_keys(json_str, keys: list[str]) -> bool:
+        """
+        Check if a JSON string contains all specified keys.
+        :param json_str: The JSON string to check.
+        :param keys: A list of keys to look for in the JSON.
+        :return: True if all keys are present, False otherwise.
+        """
         try:
             json_obj = json.loads(json_str)
             for key in keys:
@@ -25,6 +36,11 @@ class JsonUtils:
 
     @staticmethod
     def clean_json_apici(json_string):
+        """
+        Clean a JSON string by removing enclosing triple backticks and whitespace.
+        :param json_string: The JSON string to clean.
+        :return: The cleaned JSON string.
+        """
         # Rimuovi il prefisso '''json
         if json_string.startswith("```json"):
             json_string = json_string[len("```json"):]
