@@ -1,5 +1,6 @@
 import json
 import os
+import re
 import shutil
 import zipfile
 from dataclasses import dataclass, field, asdict
@@ -13,29 +14,6 @@ from pylizlib.core.log.pylizLogger import logger
 from pylizlib.core.os.path import random_subfolder, clear_folder_contents, clear_or_move_to_temp, duplicate_directory
 
 
-# S = TypeVar("S")
-
-
-# class CatalogueInterface(ABC, Generic[S]):
-#
-#     def __init__(self, path_catalogue: Path):
-#         self.__setup_catalogue(path_catalogue)
-#
-#     def __setup_catalogue(self, path_catalogue: Path):
-#         path_catalogue.mkdir(parents=True, exist_ok=True)
-#         self.path_catalogue = path_catalogue
-#
-#     def update_catalogue_path(self, new_path: Path):
-#         self.__setup_catalogue(new_path)
-#
-#     @abstractmethod
-#     def add(self, data: S) -> S:
-#         pass
-#
-#     @abstractmethod
-#     def get_all(self) -> list[S]:
-#         pass
-
 
 @dataclass
 class SnapDirAssociation:
@@ -58,11 +36,11 @@ class SnapDirAssociation:
 
 
     @staticmethod
-    def gen_random(source_folder_for_choices: Path):
+    def gen_random(source_folder_for_choices: Path, folder_id_length: int = 4) -> 'SnapDirAssociation':
         return SnapDirAssociation(
             index=SnapDirAssociation.next_index(),
             original_path=random_subfolder(source_folder_for_choices).__str__(),
-            folder_id=gen_random_string(4)
+            folder_id=gen_random_string(folder_id_length)
         )
 
     @staticmethod
@@ -88,6 +66,17 @@ class SnapEditType(Enum):
     REMOVE_DIR = "Remove"
 
 
+class SnapshotSortKey(Enum):
+    ID = "id"
+    NAME = "name"
+    DESCRIPTION = "desc"
+    AUTHOR = "author"
+    DATE_CREATED = "date_created"
+    DATE_MODIFIED = "date_modified"
+    DATE_LAST_USED = "date_last_used"
+    DATE_LAST_MODIFIED = "date_last_modified"
+
+
 @dataclass
 class SnapEditAction:
     action_type: SnapEditType
@@ -97,6 +86,29 @@ class SnapEditAction:
     directory_name_to_remove: str = ""
 
 
+@dataclass
+class SnapshotSettings:
+    json_filename: str = "snapshot.json"
+    backup_path: Path | None = None
+    backup_pre_install: bool = False
+    backup_pre_modify: bool = False
+    backup_pre_delete: bool = False
+    install_with_everyone_full_control: bool = True
+    snap_id_length: int = 20
+    folder_id_length: int = 6
+
+    @property
+    def bck_before_install_enabled(self) -> bool:
+        return self.backup_pre_install and self.backup_path is not None
+
+    @property
+    def bck_before_modify_enabled(self) -> bool:
+        return self.backup_pre_modify and self.backup_path is not None
+
+    @property
+    def bck_before_delete_enabled(self) -> bool:
+        return self.backup_pre_delete and self.backup_path is not None
+
 
 @dataclass
 class Snapshot:
@@ -195,7 +207,7 @@ class SnapshotUtils:
         dirs = SnapDirAssociation.gen_random_list(3, source_folder_for_choices)
         return Snapshot(
             id=gen_random_string(id_length),
-            name="Snapshot " + gen_random_string(5),
+            name="Snapshot " + gen_random_string(id_length),
             desc="Randomly generated snapshot",
             author="User",
             directories=dirs,
@@ -252,6 +264,45 @@ class SnapshotUtils:
 
         return edits
 
+    @staticmethod
+    def sort_snapshots(
+        snapshots: list[Snapshot],
+        sort_by: 'SnapshotSortKey',
+        reverse: bool = False
+    ) -> list[Snapshot]:
+        """
+        Sorts a list of Snapshot objects by a specified key.
+        Snapshots with a None value for the key are placed at the end.
+        String comparison is case-insensitive.
+
+        Args:
+            snapshots: The list of Snapshots to sort.
+            sort_by: The key to sort by, as a SnapshotSortKey enum member.
+            reverse: If True, sorts in descending order.
+
+        Returns:
+            A new list containing the sorted Snapshots.
+        """
+        key_attr = sort_by.value
+        snaps_with_value = []
+        snaps_with_none = []
+
+        for snap in snapshots:
+            if getattr(snap, key_attr) is None:
+                snaps_with_none.append(snap)
+            else:
+                snaps_with_value.append(snap)
+
+        def get_key(snapshot: Snapshot):
+            value = getattr(snapshot, key_attr)
+            if isinstance(value, str):
+                return value.lower()
+            return value
+
+        sorted_snaps = sorted(snaps_with_value, key=get_key, reverse=reverse)
+
+        return sorted_snaps + snaps_with_none
+
 
 
 
@@ -309,13 +360,13 @@ class SnapshotManager:
             self,
             snapshot: Snapshot,
             catalogue_path: Path,
-            json_filename: str = "snapshot.json"
+            settings: SnapshotSettings = SnapshotSettings(),
     ):
         self.snapshot = snapshot
-        self.json_filename = json_filename
+        self.settings = settings
         self.path_catalogue = catalogue_path
         self.path_snapshot = SnapshotUtils.get_snapshot_path(self.snapshot.folder_name, self.path_catalogue)
-        self.path_snapshot_json = SnapshotUtils.get_snapshot_json_path(self.snapshot.folder_name, self.path_catalogue, self.json_filename)
+        self.path_snapshot_json = SnapshotUtils.get_snapshot_json_path(self.snapshot.folder_name, self.path_catalogue, self.settings.json_filename)
 
     def __save_json(self):
         SnapshotSerializer.to_json(self.snapshot, self.path_snapshot_json)
@@ -351,7 +402,7 @@ class SnapshotManager:
         new_dir = SnapDirAssociation(
             index=SnapDirAssociation.next_index(),
             original_path=destination_path.as_posix(),
-            folder_id=gen_random_string(4)
+            folder_id=gen_random_string(self.settings.folder_id_length)
         )
         new_dir.copy_install_to(self.path_snapshot)
         self.snapshot.directories.append(new_dir)
@@ -395,11 +446,11 @@ class SnapshotManager:
         if not self.path_snapshot.exists():
             raise FileNotFoundError(f"The snapshot path {self.path_snapshot} does not exist.")
         new_snap = self.snapshot
-        new_snap.id = gen_random_string(10)
+        new_snap.id = gen_random_string(self.settings.snap_id_length)
         new_snap.name = self.snapshot.name + " Copy"
         new_snap.date_created = datetime.now()
         new_snap_path = SnapshotUtils.get_snapshot_path(new_snap.folder_name, self.path_catalogue)
-        new_snap_json_path = SnapshotUtils.get_snapshot_json_path(new_snap.folder_name, self.path_catalogue, self.json_filename)
+        new_snap_json_path = SnapshotUtils.get_snapshot_json_path(new_snap.folder_name, self.path_catalogue, self.settings.json_filename)
         duplicate_directory(self.path_snapshot, new_snap_path, "")
         SnapshotSerializer.to_json(new_snap, new_snap_json_path)
 
@@ -519,25 +570,10 @@ class SnapshotCatalogue:
     def __init__(
             self,
             path_catalogue: Path,
-            snapshot_json_filename: str = "snapshot.json",
-            backup_path: Path | None = None,
-            backup_pre_install: bool = False,
-            backup_pre_modify: bool = False,
-            backup_pre_delete: bool = False,
-            install_with_everyone_full_control: bool = True,
+            settings: SnapshotSettings = SnapshotSettings(),
     ):
         self.path_catalogue = path_catalogue
-        self.snapshot_json_filename = snapshot_json_filename
-        self.backup_path = backup_path
-        self.backup_pre_install = backup_pre_install
-        self.backup_pre_modify = backup_pre_modify
-        self.backup_pre_delete = backup_pre_delete
-        self.install_with_everyone_full_control = install_with_everyone_full_control
-
-        self.backup_pre_install_enabled = backup_pre_install and backup_path is not None
-        self.backup_pre_modify_enabled = backup_pre_modify and backup_path is not None
-        self.backup_pre_delete_enabled = backup_pre_delete and backup_path is not None
-
+        self.settings = settings
         self.path_catalogue.mkdir(parents=True, exist_ok=True)
 
     def set_catalogue_path(self, new_path: Path):
@@ -545,13 +581,13 @@ class SnapshotCatalogue:
         self.path_catalogue.mkdir(parents=True, exist_ok=True)
 
     def add(self, snap: Snapshot):
-        snap_manager = SnapshotManager(snap, self.path_catalogue, self.snapshot_json_filename)
+        snap_manager = SnapshotManager(snap, self.path_catalogue, self.settings)
         snap_manager.create()
 
     def delete(self, snap: Snapshot):
-        snap_manager = SnapshotManager(snap, self.path_catalogue, self.snapshot_json_filename)
-        if self.backup_pre_delete_enabled:
-            snap_manager.backup_snap_directory(self.backup_path)
+        snap_manager = SnapshotManager(snap, self.path_catalogue, self.settings)
+        if self.settings.bck_before_delete_enabled:
+            snap_manager.backup_snap_directory(self.settings.backup_path)
         snap_manager.delete()
 
     def get_all(self) -> list[Snapshot]:
@@ -559,7 +595,7 @@ class SnapshotCatalogue:
         snapshots: list[Snapshot] = []
         for current_dir in self.path_catalogue.iterdir():
             if current_dir.is_dir():
-                snap = SnapshotUtils.get_snapshot_from_path(current_dir, self.snapshot_json_filename)
+                snap = SnapshotUtils.get_snapshot_from_path(current_dir, self.settings.json_filename)
                 if snap is not None:
                     snapshots.append(snap)
         return snapshots
@@ -576,9 +612,9 @@ class SnapshotCatalogue:
         self.update_snapshot_by_edits(new, edits)
 
     def update_snapshot_by_edits(self, snap: Snapshot, edits: list[SnapEditAction]):
-        snap_manager = SnapshotManager(snap, self.path_catalogue, self.snapshot_json_filename)
-        if self.backup_pre_modify_enabled:
-            snap_manager.backup_snap_directory(self.backup_path)
+        snap_manager = SnapshotManager(snap, self.path_catalogue, self.settings)
+        if self.settings.bck_before_modify_enabled:
+            snap_manager.backup_snap_directory(self.settings.backup_path)
         snap_manager.update_json_base_fields()
         snap_manager.update_json_data_fields()
         snap_manager.update_from_actions_list(edits)
@@ -587,14 +623,14 @@ class SnapshotCatalogue:
         snap = self.get_by_id(snap_id)
         if snap is None:
             raise ValueError(f"No snapshot found with ID {snap_id}")
-        snap_manager = SnapshotManager(snap, self.path_catalogue, self.snapshot_json_filename)
+        snap_manager = SnapshotManager(snap, self.path_catalogue, self.settings)
         snap_manager.duplicate()
 
     def install(self, snap: Snapshot):
-        snap_manager = SnapshotManager(snap, self.path_catalogue, self.snapshot_json_filename)
-        if self.backup_pre_install_enabled:
-            snap_manager.backup_associated("preinstall", self.backup_path)
-        snap_manager.install(self.install_with_everyone_full_control)
+        snap_manager = SnapshotManager(snap, self.path_catalogue, self.settings)
+        if self.settings.bck_before_install_enabled:
+            snap_manager.backup_associated("preinstall", self.settings.backup_path)
+        snap_manager.install(self.settings.install_with_everyone_full_control)
 
     def exists(self, snap_id: str) -> bool:
         return self.get_by_id(snap_id) is not None
@@ -605,5 +641,114 @@ class SnapshotCatalogue:
         return SnapshotUtils.get_snapshot_path(snap.folder_name, self.path_catalogue)
 
 
+@dataclass
+class SnapshotSearchResult:
+    """
+    Rappresenta un singolo risultato di ricerca all'interno di un file di snapshot.
+    """
+    file_path: str
+    searched_text: str
+    line_number: int
 
 
+class SnapshotSearcher:
+    """
+    Cerca un contenuto testuale all'interno dei file degli snapshot gestiti da uno SnapshotCatalogue.
+    """
+
+    def __init__(self, catalogue: SnapshotCatalogue):
+        """
+        Inizializza lo SnapshotSearcher.
+
+        Args:
+            catalogue: Lo SnapshotCatalogue in cui cercare.
+        """
+        self.catalogue = catalogue
+
+    def search_text_snap(self, snapshot: Snapshot, text: str) -> list[SnapshotSearchResult]:
+        """
+        Cerca un dato testo in tutti i file di testo in un singolo snapshot.
+        """
+        results: list[SnapshotSearchResult] = []
+        snapshot_path = self.catalogue.get_snap_directory_path(snapshot)
+        if not snapshot_path or not snapshot_path.is_dir():
+            return results
+
+        for dir_assoc in snapshot.directories:
+            copied_dir_path = snapshot_path.joinpath(dir_assoc.directory_name)
+            if not copied_dir_path.is_dir():
+                continue
+            for file_path in copied_dir_path.rglob('*'):
+                if not file_path.is_file():
+                    continue
+                try:
+                    with file_path.open('r', encoding='utf-8') as f:
+                        for i, line in enumerate(f, 1):
+                            if text in line:
+                                results.append(SnapshotSearchResult(
+                                    file_path=str(file_path),
+                                    searched_text=text,
+                                    line_number=i
+                                ))
+                except UnicodeDecodeError:
+                    logger.debug(f"Skipping binary file during search: {file_path}")
+                except Exception as e:
+                    logger.warning(f"Error reading file {file_path} during search: {e}")
+        return results
+
+    def search_regex_snap(self, snapshot: Snapshot, regex_pattern: str) -> list[SnapshotSearchResult]:
+        """
+        Cerca un dato pattern regex in tutti i file di testo in un singolo snapshot.
+        """
+        results: list[SnapshotSearchResult] = []
+        try:
+            compiled_regex = re.compile(regex_pattern)
+        except re.error as e:
+            logger.error(f"Invalid regex pattern provided: {e}")
+            return results
+
+        snapshot_path = self.catalogue.get_snap_directory_path(snapshot)
+        if not snapshot_path or not snapshot_path.is_dir():
+            return results
+
+        for dir_assoc in snapshot.directories:
+            copied_dir_path = snapshot_path.joinpath(dir_assoc.directory_name)
+            if not copied_dir_path.is_dir():
+                continue
+            for file_path in copied_dir_path.rglob('*'):
+                if not file_path.is_file():
+                    continue
+                try:
+                    with file_path.open('r', encoding='utf-8') as f:
+                        for i, line in enumerate(f, 1):
+                            if compiled_regex.search(line):
+                                results.append(SnapshotSearchResult(
+                                    file_path=str(file_path),
+                                    searched_text=regex_pattern,
+                                    line_number=i
+                                ))
+                except UnicodeDecodeError:
+                    logger.debug(f"Skipping binary file during search: {file_path}")
+                except Exception as e:
+                    logger.warning(f"Error reading file {file_path} during search: {e}")
+        return results
+
+    def search_text_global(self, text: str) -> list[SnapshotSearchResult]:
+        """
+        Cerca un dato testo in tutti i file di testo attraverso tutti gli snapshot nel catalogo.
+        """
+        results: list[SnapshotSearchResult] = []
+        snapshots = self.catalogue.get_all()
+        for snapshot in snapshots:
+            results.extend(self.search_text_snap(snapshot, text))
+        return results
+
+    def search_regex_global(self, regex_pattern: str) -> list[SnapshotSearchResult]:
+        """
+        Cerca un dato pattern regex in tutti i file di testo attraverso tutti gli snapshot nel catalogo.
+        """
+        results: list[SnapshotSearchResult] = []
+        snapshots = self.catalogue.get_all()
+        for snapshot in snapshots:
+            results.extend(self.search_regex_snap(snapshot, regex_pattern))
+        return results
\ No newline at end of file
